c     Copyright (C) 2008-2009 Vincent Eymet
c
c     Planet_EMC is free software; you can redistribute it and/or modify
c     it under the terms of the GNU General Public License as published by
c     the Free Software Foundation; either version 3, or (at your option)
c     any later version.

c     Planet_EMC is distributed in the hope that it will be useful,
c     but WITHOUT ANY WARRANTY; without even the implied warranty of
c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c     GNU General Public License for more details.

c     You should have received a copy of the GNU General Public License
c     along with Planet_EMC; if not, see <http://www.gnu.org/licenses/>
c
      subroutine surface_emission(dim,app,
     &     r1,r2,theta1,theta2,phi1,phi2,
     &     ca,cb,cc,
     &     surface,P,u,pdf)
      implicit none
      include 'param.inc'
      include 'max.inc'
c
c     Purpose: to compute the emission direction "u" and emission pdf,
c     for a surface emission
c
c     Inputs:
c       + dim: dimension of the vectors
c       + app: if equal to 1, ground surface must be computed as the real surface at ground level
c       + r1, theta1, phi1, r2, theta2, phi2: limits of the spherical box
c       + ca,cb,cc: a, b and c coefficients for each quarter
c       + surface: surface type [1,6]
c
c     Outputs:
c       + P: (r,theta,phi) spherical coordinates of the emission point
c       + u: emission direction
c       + pdf: emission pdf
c

c     inputs
      integer dim,app
      double precision r1,r2,theta1,theta2,phi1,phi2
      double precision ca(1:4)
      double precision cb(1:4)
      double precision cc(1:4)
      integer surface
c     outputs
      double precision P(1:Nmat_mx)
      double precision u(1:Nmat_mx)
      double precision pdf
c     temp
      double precision n(1:Nmat_mx)
      double precision alpha,prod
      character*100 label
c     Debug
c      integer strlen
c      integer i,j,out
c     Debug

      label='subroutine surface_emission'
      call test_box(r1,r2,theta1,theta2,phi1,phi2,label)
      call test_surface(surface,label)

c     chosing emission point coordinates
      call surface_emission_position(r1,r2,theta1,theta2,phi1,phi2,
     &     surface,P)
      if ((surface.eq.1).and.(app.eq.1)) then
         call ground_normal(dim,P,theta1,theta2,
     &        phi1,phi2,ca,cb,cc,n)
      else
         call inside_normal(surface,P,n)
      endif
      call diffuse_emission(dim,n,u,pdf)

      return
      end



      subroutine diffuse_emission(dim,n,u,pdf)
      implicit none
      include 'max.inc'
      include 'param.inc'
c
c     Purpose: to choose isotropically a propagation direction on the
c     2*pi hemisphere defined by normal direction "n"
c
c     routine "surface_isotrope_angle" has been designed so that the integral
c     over 2*pi (one hemisphere) of u.n (with u chosen according to the routine)
c     is pi. One would only have to use the present routine to choose an emission direction u
c     according to pdf=1/(2*pi), and the integral of B(T)*u.n/pdf (the emitted flux) is pi*B(T).
c     Example of code that would use the present function:
c      sum=0.0D+0
c      sum2=0.0D+0
c      n(1)=0.0D+0
c      n(2)=0.0D+0
c      n(3)=1.0D+0
c      do i=1,Nevent
c         call diffuse_emission(dim,n,u1,pdf)
c         call scalar_product(dim,n,u1,prod)
c         ev=prod/pdf
c         sum=sum+ev
c         sum2=sum2+ev**2.0D+0
c      enddo
c      call statis(Nevent,sum,sum2,moy,var,inc)
c      write(*,*) 'moy=',moy,' +/-',inc
c      stop
c
c     Result:
c
c vincent@pan{/home/vincent/labo/Monte-Carlo/planet_EMC}[533] > mpirun -np 2 planet.exe
c Initializing...
c ...done
c Sending data to all processes...
c ...done
c Now computing...
c moy=  3.13884208 +/-  0.0180491191
c
c
c     Inputs:
c       + dim: dimension of vectors
c       + n: local normal
c
c     Output:
c       + u: propagation direction
c       + pdf: pdf associated with this choice of "u"
c

      integer dim
      double precision n(1:Nmat_mx)
      double precision u(1:Nmat_mx)
      double precision pdf
      double precision alpha,prod
      double precision prod_min
      prod_min=1.0D-3

      prod=-1.0D+0
      do while (prod.lt.prod_min)
c     chosing emission angle / normal: alpha [0,pi/2]
         call surface_isotrope_angle(alpha,pdf)
c     computing emission direction coordinates
         call new_direction(dim,n,alpha,u)
         call scalar_product(dim,n,u,prod)
      enddo

      return
      end



      subroutine surface_emission_position(r1,r2,theta1,theta2,
     &     phi1,phi2,surface,P)
      implicit none
      include 'param.inc'
      include 'max.inc'
c
c     Purpose: to sample uniformly the emission position
c     on a surface.
c
c     Inputs:
c       + r1, theta1, phi1, r2, theta2, phi2: limits of the spherical box
c       + surface: surface type [1,6]
c
c     Outputs:
c       + P: (r,theta,phi) spherical coordinates of the emission point
c

c     inputs
      double precision r1,r2,theta1,theta2,phi1,phi2
      integer surface
c     output
      double precision P(1:Nmat_mx)

      if ((surface.eq.1).or.(surface.eq.2)) then
         if (surface.eq.1) then
            P(1)=r1
         else
            P(1)=r2
         endif
         call uniform(theta1,theta2,P(2))
         call uniform(phi1,phi2,P(3))
      else if ((surface.eq.3).or.(surface.eq.4)) then
         call uniform(r1,r2,P(1))
         call uniform(theta1,theta2,P(2))
         if (surface.eq.3) then
            P(3)=phi1
         else
            P(3)=phi2
         endif
      else if ((surface.eq.5).or.(surface.eq.6)) then
         call uniform(r1,r2,P(1))
         if (surface.eq.5) then
            P(2)=theta1
         else
            P(2)=theta2
         endif
         call uniform(phi1,phi2,P(3))
      endif

      return
      end



      subroutine angular_physical_pdf(dim,u,n,pdf0)
      implicit none
      include 'max.inc'
      include 'param.inc'
c     
c     Purpose: to generate the physical pdf of a surface emission along emission direction "u"
c     and local normal "n"
c
c     Inputs:
c       + dim: dimension of vectors
c       + u: emission direction from a surface
c       + n: local normal on that surface
c
c     Outputs:
c       + pdf0: probability density function of choosing direction "u"
c              according to the physics
c

c     inputs
      integer dim
      double precision u(1:Nmat_mx)
      double precision n(1:Nmat_mx)
c     outputs
      double precision pdf0
c     temp
      double precision prod

      call scalar_product(dim,n,u,prod)
      pdf0=prod/pi

      return
      end



      subroutine choose_surface(r1,r2,theta1,theta2,phi1,phi2,
     &     surface,pdf)
      implicit none
      include 'max.inc'
c
c     Purpose: to chose a surface among the 6 surfaces of the box
c     when emitting from the surface of a box
c
c     Inputs:
c       + r1, theta1, phi1, r2, theta2, phi2: limits of the spherical box
c
c     Outputs
c       + surface: index of the surface that was selected [1,6]
c       + pdf: pdf associated to this choice
c

c     inputs
      double precision r1,r2,theta1,theta2,phi1,phi2
c     outputs
      integer surface
      double precision pdf
c     temp
      double precision R
      double precision surface_r
      double precision surface_theta
      double precision surface_phi
      integer i,int,intf,strlen
      double precision S(1:6),Stotal
      double precision cumul(0:6)
      character*100 label

      label='subroutine choose_surface'
      call test_box(r1,r2,theta1,theta2,phi1,phi2,label)

      S(1)=surface_r(r1,theta1,theta2,phi1,phi2)
      S(2)=surface_r(r2,theta1,theta2,phi1,phi2)
      S(3)=surface_theta(r1,r2,theta1,theta2)
      S(4)=surface_theta(r1,r2,theta1,theta2)
      S(5)=surface_phi(theta1,r1,r2,phi1,phi2)
      S(6)=surface_phi(theta2,r1,r2,phi1,phi2)
      cumul(0)=0.0D+0
      do i=1,6
         cumul(i)=cumul(i-1)+S(i)
      enddo
      Stotal=cumul(6)
      do i=0,6
         cumul(i)=cumul(i)/Stotal
      enddo

      intf=0
      call random_gen(R)
      do i=1,6
         if ((R.ge.cumul(i-1)).and.(R.le.cumul(i))) then
            intf=1
            int=i
            goto 123
         endif
      enddo
 123  continue
      if (intf.eq.0) then
         write(*,*) 'Error from ',label(1:strlen(label)),':'
         write(*,*) 'cumulative could not be inverted for:'
         write(*,*) 'R=',R
         do i=0,6
            write(*,*) 'cumul(',i,')=',cumul(i)
         enddo
         stop
      endif
      surface=int
      pdf=S(surface)/Stotal

      return
      end




      subroutine inside_normal(surface,P,n)
      implicit none
      include 'param.inc'
      include 'max.inc'
c
c     Purpose: to provide the coordinates of the local normal "n"
c     THAT IS DIRECTED TO THE INSIDE OF THE CELL
c     and the local rotation axe "j1"
c     to the surface, at the specified point
c
c     Inputs:
c       + surface: surface type [1,6]
c       + P: (r,theta,phi) spherical coordinates of the point on the surface
c
c     Outputs:
c       + n: coordinates of the local normal, toward the inside of the cell
c

c     input
      integer surface
      double precision P(1:Nmat_mx)
c     outputs
      double precision n(1:Nmat_mx)
c     temp
      double precision axe(1:Nmat_mx)
      double precision u(1:Nmat_mx)
      double precision alpha

      if (surface.eq.1) then
         n(1)=dcos(P(2))*dcos(P(3))
         n(2)=dcos(P(2))*dsin(P(3))
         n(3)=dsin(P(2))
      else if (surface.eq.2) then
         n(1)=-dcos(P(2))*dcos(P(3))
         n(2)=-dcos(P(2))*dsin(P(3))
         n(3)=-dsin(P(2))
      else if (surface.eq.3) then
         n(1)=-dsin(P(3))
         n(2)=dcos(P(3))
         n(3)=0.0D+0
      else if (surface.eq.4) then
         n(1)=dsin(P(3))
         n(2)=-dcos(P(3))
         n(3)=0.0D+0
      else if (surface.eq.5) then
         u(1)=dcos(P(2))*dcos(P(3))
         u(2)=dcos(P(2))*dsin(P(3))
         u(3)=dsin(P(2))
         axe(1)=-dsin(P(3))
         axe(2)=dcos(P(3))
         axe(3)=0.0D+0
         alpha=-pi/2.0D+0
         call rotation(3,u,alpha,axe,n)
      else if (surface.eq.6) then
         u(1)=dcos(P(2))*dcos(P(3))
         u(2)=dcos(P(2))*dsin(P(3))
         u(3)=dsin(P(2))
         axe(1)=-dsin(P(3))
         axe(2)=dcos(P(3))
         axe(3)=0.0D+0
         alpha=pi/2.0D+0
         call rotation(3,u,alpha,axe,n)
      endif

      return
      end
      


      subroutine test_surface(surface,label)
      implicit none
      integer surface,strlen
      character*100 label

      if ((surface.lt.1).or.(surface.gt.6)) then
         write(*,*) 'Error from ',label(1:strlen(label)),':'
         write(*,*) 'Incorrect value for:'
         write(*,*) 'surface=',surface
         stop
      endif

      return
      end



      subroutine surface_isotrope_angle_ori(alpha,pdf)
      implicit none
      include 'max.inc'
      include 'param.inc'
c
c     Purpose: to pick up an emission angle [0,pi/2] from a boundary
c     according to pdf(w)=1/(2pi)
c
c     Outputs:
c       + alpha: emission angle [0,pi/2]
c       + pdf: probability density function corresponding to the chosen angle
c

c     output
      double precision alpha,pdf
c     temp
      double precision R
      integer strlen
      character*100 label
      label='subroutine surface_isotrope_angle_ori'

      call random_gen(R)
      alpha=dacos(R)
      pdf=1.0D+0/(2.0D+0*pi)
c     Tests
      if ((alpha.lt.0.0D+0).or.(alpha.gt.pi/2.0D+0)) then
         write(*,*) 'Error from surface_isotrope_angle:'
         write(*,*) 'alpha=',alpha
         write(*,*) 'is not between 0 and pi/2'
         write(*,*) 'R=',R
         stop
      endif
c     Tests

      return
      end




      subroutine surface_isotrope_angle(alpha,pdf)
      implicit none
      include 'max.inc'
      include 'param.inc'
c
c     Purpose: to pick up an emission angle [0,pi/2] from a boundary
c     according to pdf(w)=u.n/pi
c
c     Outputs:
c       + alpha: emission angle [0,pi/2]
c       + pdf: probability density function corresponding to the chosen angle
c

c     output
      double precision alpha,pdf
c     temp
      double precision R
      integer strlen
      character*100 label
      label='subroutine surface_isotrope_angle'

      call random_gen(R)
      alpha=0.5D+0*dacos(1.0D+0-2.0D+0*R)
      pdf=dcos(alpha)/pi
c     Tests
      if ((alpha.lt.0.0D+0).or.(alpha.gt.pi/2.0D+0)) then
         write(*,*) 'Error from surface_isotrope_angle:'
         write(*,*) 'alpha=',alpha
         write(*,*) 'is not between 0 and pi/2'
         write(*,*) 'R=',R
         stop
      endif
c     Tests

      return
      end
