c     Copyright (C) 2009 Vincent Eymet
c
c     "satellite" is free software; you can redistribute it and/or modify
c     it under the terms of the GNU General Public License as published by
c     the Free Software Foundation; either version 3, or (at your option)
c     any later version.

c     This software is distributed in the hope that it will be useful,
c     but WITHOUT ANY WARRANTY; without even the implied warranty of
c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c     GNU General Public License for more details.

c     You should have received a copy of the GNU General Public License
c     along with the present software; if not, see <http://www.gnu.org/licenses/>
c
      subroutine solid_angles(dim,Ntheta,Nphi,theta,phi,Nb_gas,
     &     Psat,Pground,
     &     rspace,op_alpha,
     &     Nint_sectors,int_sectors,sector,
     &     solar_flux_toa,solar_inc_flux_toa,
     &     Pc_1s,Pc_tot_1s,inc_Pc_1s,inc_Pc_tot_1s)
      implicit none
      include 'mpif.h'
      include 'max.inc'
      include 'param.inc'
c
c     Purpose: to compute
c     Pc_1s: the power signal (as a function of narrowband index) and
c     Pc_tot_1s: the total power
c     received by the satellite detector,
c     from a single TOA angular sector.
c
c     Inputs:
c       + dim: dimension of vectors
c       + Ntheta,Nphi: number of angular sectors over the latitude / over the longitude
c       + theta: latitude angular discretization
c       + phi: longitude angular discretization
c       + Nb_gas: number of gas spectral narrowbands
c       + Psat: altitude/latitude/longitude of satellite
c       + Pground: altitude/latitude/longitude of the point (at ground level)
c       + rspace: radius at space level in meters
c       + op_alpha: angle of the observation cone
c       + Nint_sectors: number of angular sectors that are intersected
c       + int_sectors: angular sectors that are intersected at space level (even partially)
c         by the observation cone
c         - int_sectors(i,1): itheta for sector index i
c         - int_sectors(i,2): iphi for sector index i
c         - 3 and 4: u_itheta and u_iphi in which the satellite is for sector index i
c       + sector: index of the angular sector results have to be computed for
c
c     Outputs:
c       + Pc_1s: power received by the satellite (Watts per square meter of detector, in each narrowband)
c       + Pc_tot_1s: total power received by the detector (Watts per square meter of detector)
c       + inc_Pc_1s and inc_Pc_tot_1s: uncertainties over Pc_1s and Pc_tot_1s
c

c     input
      integer dim,Nphi,Ntheta,Nb_gas
      double precision theta(0:Ntheta_mx)
      double precision phi(0:Nphi_mx)
      double precision Psat(1:Nmat_mx)
      double precision Pground(1:Nmat_mx)
      double precision rspace,op_alpha
      integer Nint_sectors
      integer int_sectors(1:Ntheta_mx*Nphi_mx,1:4)
      integer sector
      double precision solar_flux_toa(1:Ntheta_mx,1:Nphi_mx,
     &     1:Ntheta_mx,1:Nphi_mx,Nb_gas_mx) ! (W/m)
      double precision solar_inc_flux_toa(1:Ntheta_mx,1:Nphi_mx,
     &     1:Ntheta_mx,1:Nphi_mx,Nb_gas_mx) ! (W/m)
c     output
      double precision Pc_1s(1:Nb_gas_mx),Pc_tot_1s
      double precision inc_Pc_1s(1:Nb_gas_mx),inc_Pc_tot_1s
c     temp
      integer Nss
      integer Nint,los
      integer itheta,iphi,band
      integer it,ip
      integer u_itheta,u_iphi
      double precision theta1,theta2
      double precision dtheta,dphi
      double precision phi1,phi2
      double precision thetas,phis
      double precision dwc,w
      double precision S,dS
      double precision dsc
      double precision sp,sps
      double precision Ps(1:Nmat_mx)
      double precision ns(1:Nmat_mx)
      double precision u_obs(1:Nmat_mx)
      double precision u(1:Nmat_mx)
      double precision us(1:Nmat_mx)
      double precision Pint(1:Nmat_mx)
      double precision surface_r
      double precision P,inc_P,ksi
      integer j
      integer strlen
      character*100 label
c     MPICH
      integer ierr,errorcode,np,pindex
      integer stat(MPI_STATUS_SIZE)
c     MPICH
      label='subroutine solid_angles'

c     MPICH
c     Get 'np', the number of processes
      call MPI_COMM_SIZE(MPI_COMM_WORLD,np,ierr)
c     Get 'pindex', the index of the current process
      call MPI_COMM_RANK(MPI_COMM_WORLD,pindex,ierr)
c     MPICH

      Nint=1000 ! number of intervals used for (theta,phi) sub-discretization
      Nss=0 ! number of intersected sub-sectors
c     Debug
c      write(*,*) 'op_alpha=',op_alpha
c      write(*,*) 'cos(op_alpha)=',dcos(op_alpha)
c     Debug
c     compute observation direction
      call observation_direction(dim,Psat,Pground,u_obs)
c     initialization of power received by the detector
      do band=1,Nb_gas
         Pc_1s(band)=0.0D+0
         inc_Pc_1s(band)=0.0D+0
      enddo ! band

      itheta=int_sectors(sector,1)
      iphi=int_sectors(sector,2)
c     S is the total surface at space level over the (itheta,iphi) angular sector
      S=surface_r(rspace,
     &     theta(itheta-1),theta(itheta),
     &     phi(iphi-1),phi(iphi))
c     Debug
c      write(*,*) 'S=',S
c     Debug
c     dtheta is the latitude variation over a sub-sector
      dtheta=(theta(itheta)-theta(itheta-1))/dble(Nint)
c     dphi is the longitude variation over a sub-sector
      dphi=(phi(iphi)-phi(iphi-1))/dble(Nint)
c     dS is the surface at space level of a small angular sector
      dS=S/dble(Nint**2)
      do it=1,Nint
c     (theta1,theta2) are the limits in latitude of the sub-sector
         theta1=theta(itheta-1)+dtheta*dble(it-1)
         theta2=theta(itheta-1)+dtheta*dble(it)
c     thetas is the central latitude of the sub-sector
         thetas=(theta1+theta2)/2.0D+0
         do ip=1,Nint
c     (phi1,phi2) are the limits in longitude of the sub-sector
            phi1=phi(iphi-1)+dphi*dble(ip-1)
            phi2=phi(iphi-1)+dphi*dble(ip)
c     phis is the central longitude of the sub-sector
            phis=(phi1+phi2)/2.0D+0
c     Ps is the central position of the sub-sector
            Ps(1)=rspace
            Ps(2)=thetas
            Ps(3)=phis
c     ns is the outside normal at space level, at the center of the sub-sector
            call inside_normal(1,Ps,ns)
c     --------------------------------------------------------------------------------------
c     looking for intersection position Pint between line of sight and sphere at space level
            call check_los(dim,Ps,Psat,los)
            if (los.eq.1) then
c     in that case, there is an emission from the local sub-sector possible towards the satellite
c     identification of the sector the satellite is seen in from Ps
               call observation_direction(dim,Ps,Psat,u)
               call sort_direction(dim,Ntheta,Nphi,theta,phi,u,
     &              u_itheta,u_iphi)
c     us is the direction of the line of sight between the satellite and Ps
               call observation_direction(dim,Psat,Ps,us)
c     sp: scalar product between line of sight and observation direction
               call scalar_product(dim,u_obs,us,sp)
c     if sp is lower than the cosine of the observation cone angle,
               if (sp.ge.dcos(op_alpha)) then
                  Nss=Nss+1
c     then the local sub-sector contributes to the energy received by the detector
c     dsc is the distance between the local sub-sector and the satellite
                  call distance_spher(label,Ps(1),Ps(2),Ps(3),
     &                 Psat(1),Psat(2),Psat(3),dsc)
c     sps is the scalar product between the direction of the satellite (seen from Ps)
c     and the local outgoing normal at Ps
                  call scalar_product(dim,u,ns,sps)
c     dwc is the solid angle the local sub-sector is seen in from the satellite
                  dwc=dS*sps/dsc**2.0D+0
c     w is the solid angle of the (u_itheta,u_iphi) angular sector
                  call s_angle(theta(u_itheta-1),theta(u_itheta),
     &                 phi(u_iphi-1),phi(u_iphi),w)
c     ksi is the geometric factor for the local sub-sector
                  ksi=dwc*sp/w
                  do band=1,Nb_gas
                     P=solar_flux_toa(itheta,iphi,
     &                    u_itheta,u_iphi,band)
                     inc_P=solar_inc_flux_toa(itheta,iphi,
     &                    u_itheta,u_iphi,band)
                     Pc_1s(band)=Pc_1s(band)+P*ksi
                     inc_Pc_1s(band)=inc_Pc_1s(band)+inc_P*ksi
                  enddo         ! band
               endif
            endif
c     --------------------------------------------------------------------------------------
         enddo                  ! ip
      enddo                     ! it

      Pc_tot_1s=0.0D+0
      inc_Pc_tot_1s=0.0D+0
      do band=1,Nb_gas
         Pc_tot_1s=Pc_tot_1s+Pc_1s(band)
         inc_Pc_tot_1s=inc_Pc_tot_1s+inc_Pc_1s(band)
      enddo

      return
      end



      subroutine check_los(dim,P1,P2,out)
      implicit none
      include 'max.inc'
      include 'param.inc'
c
c     To detect whether a line of sight is possible between two positions,
c     one of which is defined on a sphere
c
c     Inputs:
c       + dim: dimension of vectors
c       + P1: first position (r,theta,phi) on the sphere
c       + P2: second position (r,theta,phi) (satellite position)
c
c     Outputs:
c       + out: 1 is line of sight is possible; 0 otherwise
c

c     input
      integer dim
      double precision P1(1:Nmat_mx)
      double precision P2(1:Nmat_mx)
c     output
      integer out
c     temp
      double precision max_dist
      double precision dist
      integer intf
      double precision Pint(1:Nmat_mx)
      double precision us(1:Nmat_mx)
      double precision r
      integer strlen
      character*100 label
      label='subroutine check_los'

      out=0 ! default
      max_dist=1.0D+3 ! m

c     r is the radius of the sphere
      r=P1(1)
c     check that P2(1) is greater than r
      if (P2(1).le.r) then
         write(*,*) 'Error from ',label(1:strlen(label)),' :'
         write(*,*) 'position P2 is inside the sphere'
         stop
      endif
c     us is the direction of the line of sight between P1 and P2
      call observation_direction(dim,P2,P1,us)
c     looking for intersection position Pint between line of sight and sphere at space level
      call line_surface_r_intersect(0,dim,P2,us,
     &     r,-pi/2.0D+0,pi/2.0D+0,0.0D+0,2.0D+0*pi,
     &     intf,Pint)
      if (intf.eq.0) then
         write(*,*) 'Error from ',label(1:strlen(label)),' :'
         write(*,*) 'No intersection found'
         stop
      endif
c     computing "dist", the distance between Pint and Ps
      call distance_spher(label,P1(1),P1(2),P1(3),
     &     Pint(1),Pint(2),Pint(3),dist)
c     if "dist" is small, then Pint and Ps can be considered as equal
      if (dist.le.max_dist) then
         out=1
      endif

      return
      end

